@@ -22,14 +22,18 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: vbi.c,v 1.6 2002/10/22 04:42:40 mschimek Exp $ */
+/* $Id: vbi.c,v 1.17 2006/02/10 06:25:38 mschimek Exp $ */
 
 #include "site_def.h"
-#include "../config.h"
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdarg.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <time.h>
@@ -38,6 +42,8 @@
 #include <sys/time.h>
 #include <pthread.h>
 
+#include "misc.h"
+#include "version.h"
 #include "vbi.h"
 #include "hamm.h"
 #include "lang.h"
@@ -51,25 +57,36 @@
  *
  * @author Iaki Garca Etxebarria<br>
  * Michael H. Schimek<br>
+ * Tom Zoerner<br>
  * based on AleVT by Edgar Toernig
  *
  * @section intro Introduction
  *
  * The ZVBI library provides routines to access raw VBI sampling devices
- * (currently the Linux <a href="http://roadrunner.swansea.uk.linux.org/v4l.shtml">V4L</a>
- * and <a href="http://www.thedirks.org/v4l2/">V4L2</a> API and the
- * FreeBSD, OpenBSD, NetBSD and BSDi
+ * (currently the Linux <a href="http://linux.bytesex.org/v4l2/">V4L and
+ * and V4L2</a> API and the FreeBSD, OpenBSD, NetBSD and BSDi
  * <a href="http://telepresence.dmem.strath.ac.uk/bt848/">bktr driver</a> API
  * are supported), a versatile raw VBI bit slicer,
  * decoders for various data services and basic search,
  * render and export functions for text pages. The library was written for
  * the <a href="http://zapping.sourceforge.net">Zapping TV viewer and
  * Zapzilla Teletext browser</a>.
+ *
+ * @section feedback Feedback
+ *
+ * If you have any ideas, questions, patches or bug reports please see
+ * the README file included with the source code or visit our home page at
+ * <a href="http://zapping.sourceforge.net">http://zapping.sourceforge.net</a>.
  */
 
 /** @defgroup Basic Basic types */
 /** @defgroup Raw Raw VBI */
-/** @defgroup Service Data Service Decoder */
+/** @defgroup LowDec Low Level Decoding */
+/** @defgroup HiDec High Level Decoding */
+/**
+ * @defgroup Service Data Service Decoder
+ * @ingroup HiDec
+ */
 
 pthread_once_t vbi_init_once = PTHREAD_ONCE_INIT;
 
@@ -122,7 +139,9 @@ vbi_event_enable(vbi_decoder *vbi, int mask)
  * @param handler Event handler function.
  * @param user_data Pointer passed to the handler.
  * 
- * @deprecated Use vbi_event_handler_register() in new code.
+ * @deprecated
+ * Replaces all existing handlers with this @a handler function,
+ * ignoring @a user_data. Use vbi_event_handler_register() in new code.
  * 
  * @return
  * FALSE on failure.
@@ -185,7 +204,9 @@ vbi_event_handler_add(vbi_decoder *vbi, int event_mask,
  * @param vbi Initialized vbi decoding context.
  * @param handler Event handler function.
  * 
- * @deprecated Use vbi_event_handler_register() in new code.
+ * @deprecated
+ * This functions lacks a user_data parameter.
+ * Use vbi_event_handler_register() in new code.
  */
 void
 vbi_event_handler_remove(vbi_decoder *vbi, vbi_event_handler handler)
@@ -511,6 +532,8 @@ vbi_channel_switched(vbi_decoder *vbi, vbi_nuid nuid)
 {
 	/* XXX nuid */
 
+	nuid = nuid;
+
 	pthread_mutex_lock(&vbi->chswcd_mutex);
 
 	vbi->chswcd = 1;
@@ -812,29 +835,23 @@ vbi_decoder_new(void)
 }
 
 /**
- * @internal
- * libzvbi internal helper function.
- * Note asprintf() is a GNU libc extension.
+ * @ingroup Basic
+ *
+ * @param major Store major number here, can be NULL.
+ * @param minor Store minor number here, can be NULL.
+ * @param micro Store micro number here, can be NULL.
+ *
+ * Returns the library version defined in the libzvbi.h header file
+ * when the library was compiled.
+ *
+ * @since 0.2.5
  */
 void
-vbi_asprintf(char **errstr, char *templ, ...)
+vbi_version			(unsigned int *		major,
+				 unsigned int *		minor,
+				 unsigned int *		micro)
 {
-	char buf[512];
-	va_list ap;
-	int temp;
-
-	if (!errstr)
-		return;
-
-	temp = errno;
-
-	va_start(ap, templ);
-
-	vsnprintf(buf, sizeof(buf) - 1, templ, ap);
-
-	va_end(ap);
-
-	*errstr = strdup(buf);
-
-	errno = temp;
+	if (major) *major = VBI_VERSION_MAJOR;
+	if (minor) *minor = VBI_VERSION_MINOR;
+	if (micro) *micro = VBI_VERSION_MICRO;
 }
